@@ -2,8 +2,9 @@
 # -*- coding: utf-8 -*-
 
 import fnmatch
+import os
 from pyutilb.util import *
-from pyutilb import log
+from pyutilb import log, YamlBoot, BreakException
 import pandas as pd
 from openpyxl.utils.dataframe import dataframe_to_rows
 from openpyxl.utils.cell import column_index_from_string, get_column_letter, range_boundaries
@@ -13,6 +14,7 @@ from openpyxl.drawing.image import Image
 from ExcelBoot.db import Db
 from ExcelBoot.styleable_wrapper import StyleableWrapper
 from ExcelBoot.df_col_mapper import DfColMapper
+import ExcelBoot.plt_ext as plt
 import platform
 is_win = platform.system().lower() == 'windows'
 if is_win:
@@ -20,33 +22,13 @@ if is_win:
     pythoncom.CoInitialize()
     import win32com.client
 
-# 跳出循环的异常
-class BreakException(Exception):
-    def __init__(self, condition):
-        self.condition = condition # 跳转条件
-
 # excel操作的基于yaml的启动器
-class Boot(object):
+class Boot(YamlBoot):
 
     def __init__(self):
-        # 步骤文件所在的目录
-        self.step_dir = None
-        # 已下载过的url对应的文件，key是url，value是文件
-        self.downloaded_files = {}
+        super().__init__()
         # 动作映射函数
-        self.actions = {
-            'sleep': self.sleep,
-            'print': self.print,
-            'for': self.do_for,
-            'once': self.once,
-            'break_if': self.break_if,
-            'moveon_if': self.moveon_if,
-            'moveon_if_exist_by': self.moveon_if_exist_by,
-            'break_if_exist_by': self.break_if_exist_by,
-            'break_if_not_exist_by': self.break_if_not_exist_by,
-            'include': self.include,
-            'set_vars': self.set_vars,
-            'print_vars': self.print_vars,
+        actions = {
             'start_edit': self.start_edit,
             'end_edit': self.end_edit,
             'switch_sheet': self.switch_sheet,
@@ -67,222 +49,16 @@ class Boot(object):
             'unmerge_cells': self.unmerge_cells,
             'insert_image': self.insert_image,
             'insert_file': self.insert_file,
+            'insert_plot': self.insert_plot,
         }
-        set_var('boot', self)
-        # 当前文件
-        self.step_file = None
+        self.add_actions(actions)
+
+        # excel对象
         self.wb: Workbook = None # book
         self.ws: Worksheet = None # sheet
         self.sheet = None # sheet名
 
-    '''
-    执行入口
-    :param step_files 步骤配置文件或目录的列表
-    '''
-    def run(self, step_files):
-        for path in step_files:
-            # 1 模式文件
-            if '*' in path:
-                dir, pattern = path.rsplit(os.sep, 1)  # 从后面分割，分割为目录+模式
-                if not os.path.exists(dir):
-                    raise Exception(f'Step config directory not exist: {dir}')
-                self.run_1dir(dir, pattern)
-                return
-
-            # 2 不存在
-            if not os.path.exists(path):
-                raise Exception(f'Step config file or directory not exist: {path}')
-
-            # 3 目录: 遍历执行子文件
-            if os.path.isdir(path):
-                self.run_1dir(path)
-                return
-
-            # 4 纯文件
-            self.run_1file(path)
-
-    # 执行单个步骤目录: 遍历执行子文件
-    # :param path 目录
-    # :param pattern 文件名模式
-    def run_1dir(self, dir, pattern ='*.yml'):
-        # 遍历目录: https://blog.csdn.net/allway2/article/details/124176562
-        files = os.listdir(dir)
-        files.sort() # 按文件名排序
-        for file in files:
-            if fnmatch.fnmatch(file, pattern): # 匹配文件名模式
-                file = os.path.join(dir, file)
-                if os.path.isfile(file):
-                    self.run_1file(file)
-
-    # 执行单个步骤文件
-    # :param step_file 步骤配置文件路径
-    # :param include 是否inlude动作触发
-    def run_1file(self, step_file, include = False):
-        # 获得步骤文件的绝对路径
-        if include: # 补上绝对路径
-            if not os.path.isabs(step_file):
-                step_file = self.step_dir + os.sep + step_file
-        else: # 记录目录
-            step_file = os.path.abspath(step_file)
-            self.step_dir = os.path.dirname(step_file)
-
-        log.debug(f"Load and run step file: {step_file}")
-        # 获得步骤
-        steps = read_yaml(step_file)
-        self.step_file = step_file
-        # 执行多个步骤
-        self.run_steps(steps)
-
-    # 执行多个步骤
-    def run_steps(self, steps):
-        # 逐个步骤调用多个动作
-        for step in steps:
-            for action, param in step.items():
-                self.run_action(action, param)
-
-    '''
-    执行单个动作：就是调用动作名对应的函数
-    :param action 动作名
-    :param param 参数
-    '''
-    def run_action(self, action, param):
-        log.debug(f"handle action: {action}={param}")
-
-        is_for_action = '(' in action # 是否循环动作
-        if is_for_action: # 解析循环动作
-            action, params = parse_func(action)
-            n = params[0]
-
-        if action not in self.actions:
-            raise Exception(f'Invalid action: [{action}]')
-
-        # 调用动作对应的函数
-        func = self.actions[action]
-        if is_for_action: # 循环动作: 多加了个参数-循环变量n
-            func(param, n)
-        else:
-            func(param)
-
     # --------- 动作处理的函数 --------
-    # 当前url
-    # @property
-    # def current_url(self):
-    #     if self.driver == None:
-    #         return None
-    #     return self.driver.current_url
-
-    # 解析动作名for(n)中的n: 或数字或列表
-    def parse_for_n(self, n):
-        if n == None or n == '':
-            return None
-
-        # 1 数字
-        if n.isdigit():
-            return int(n)
-
-        # 2 变量表达式, 必须是int/list/df.Series类型
-        expr = "${" + n + "}"
-        n = replace_var(expr, False)
-
-        # fix bug: pd.Series == None 居然返回pd.Series
-        if isinstance(n, pd.Series):
-            return n
-        if n == None or not (isinstance(n, (list, tuple, set, int))):
-            raise Exception(f'Variable in for({n}) parentheses must be int or list or pd.Series type')
-        return n
-
-    # for循环
-    # :param steps 每个迭代中要执行的步骤
-    # :param n 循环次数/循环列表变量名
-    def do_for(self, steps, n = None):
-        n = self.parse_for_n(n)
-        label = f"for({n})"
-        # 循环次数
-        # fix bug: pd.Series == None 居然返回pd.Series
-        n_null = (not isinstance(n, pd.Series)) and n == None
-        if n_null:
-            n = sys.maxsize # 最大int，等于无限循环次数
-            label = f"for(∞)"
-        # 循环的列表值
-        items = None
-        if isinstance(n, (list, tuple, set, pd.Series)):
-            items = n
-            n = len(items)
-        log.debug(f"-- Loop start: {label} -- ")
-        last_i = get_var('for_i', False) # 旧的索引
-        last_v = get_var('for_v', False) # 旧的元素
-        try:
-            for i in range(n):
-                # i+1表示迭代次数比较容易理解
-                log.debug(f"{i+1}th iteration")
-                set_var('for_i', i+1) # 更新索引
-                if n_null:
-                    v = None
-                else:
-                    v = items[i]
-                set_var('for_v', v) # 更新元素
-                self.run_steps(steps)
-        except BreakException as e:  # 跳出循环
-            log.debug(f"-- Loop break: {label}, break condition: {e.condition} -- ")
-        else:
-            log.debug(f"-- Loop finish: {label} -- ")
-        finally:
-            set_var('for_i', last_i) # 恢复索引
-            set_var('for_v', last_v) # 恢复元素
-
-    # 执行一次子步骤，相当于 for(1)
-    def once(self, steps):
-        self.do_for(steps, 1)
-
-    # 检查并继续for循环
-    def moveon_if(self, expr):
-        # break_if(条件取反)
-        self.break_if(f"not ({expr})")
-
-    # 跳出for循环
-    def break_if(self, expr):
-        val = eval(expr, globals(), bvars)  # 丢失本地与全局变量, 如引用不了json模块
-        if bool(val):
-            raise BreakException(expr)
-
-    # 检查并继续for循环
-    def moveon_if_exist_by(self, config):
-        self.break_if_not_exist_by(config)
-
-    # 跳出for循环
-    def break_if_not_exist_by(self, config):
-        if not self.exist_by_any(config):
-            raise BreakException(config)
-
-    # 跳出for循环
-    def break_if_exist_by(self, config):
-        if self.exist_by_any(config):
-            raise BreakException(config)
-
-    # 加载并执行其他步骤文件
-    def include(self, step_file):
-        self.run_1file(step_file, True)
-
-    # 设置变量
-    def set_vars(self, vars):
-        for k, v in vars.items():
-            v = replace_var(v)  # 替换变量
-            set_var(k, v)
-
-    # 打印变量
-    def print_vars(self, _):
-        log.info(f"Variables: {bvars}")
-
-    # 睡眠
-    def sleep(self, seconds):
-        seconds = replace_var(seconds)  # 替换变量
-        time.sleep(int(seconds))
-
-    # 打印
-    def print(self, msg):
-        msg = replace_var(msg)  # 替换变量
-        log.debug(msg)
-
     # 开始编辑excel
     def start_edit(self, file):
         if self.wb != None:
@@ -811,6 +587,19 @@ class Boot(object):
             self.reload_wb()
             self.reload_ws()
 
+    # 插入plot绘图
+    def insert_plot(self, config):
+        for bound, opt in config.items():
+            # 获得df的变量
+            var_df = opt["df"] # df变量名
+            del opt["df"]
+            df = self.get_var_DataFrame(var_df) # df
+            # 绘图
+            file = plt.plot(df, **opt)
+            # 添加图片
+            img = Image(file)
+            self.ws.add_image(img, bound)
+
 # cli入口
 def main():
     # 基于yaml的执行器
